<?php
/** 
* Load cache code
*/
require_once 'Horde/Cache.php';

/**
 * Faq_Categories:: defines an API for implementing categorie tree.
 *
 * NOTE: copied version of Merk categories
 *
 * Copyright 2006 Duck <duck@obala.net>
 *
 * See the enclosed file LICENSE for license information (BSD). If you
 * did not receive this file, see http://cvs.horde.org/co.php/faq/LICENSE.
 *
 * $Horde: faq/lib/Categories.php,v 1.42 2006/08/15 18:07:43 duck Exp $
 *
 * @author Aleksandar Markovic <mikikg@ptt.yu>
 * @package Faq
 */
class Faq_Categories extends DataTree {

    /**
     * The DataTree object which is used to store the categories.
     *
     * @var DataTree
     */
    var $_categories;

    /**
     * An array of all attributes for the categories. We store them here as an
     * so that only one trip to SQL is needed for them, and they will be
     * accessed in various places - for the category tree (requiring almost
     * all the attributes), for subcategory names and images, etc.
     *
     * @var array
     */
    var $_attributes;

    /**
     * The Cache object which is used to cache categories data
     *
     * @var cache
     */
    var $_cache;

    /**
     * Constructor
     */
    function Faq_Categories()
    {
        global $conf;

        /* Load datatree */
        if (empty($conf['datatree']['driver'])) {
            Horde::fatal(_("You must configure a Horde DataTree backend to use Faq."), __FILE__, __LINE__);
        }

        $params = array_merge(Horde::getDriverConfig('datatree', $conf['datatree']['driver']),
                              array('group' => 'faq.categories',
                                    'table' => $conf['storage']['params']['categories']['table'],
                                    'table_attributes' => $conf['storage']['params']['categories']['table_attributes']));

        $this->_categories = &DataTree::singleton($conf['datatree']['driver'], $params);

        /* Load cache */
        $this->_cache = &Horde_Cache::singleton($conf['cache']['driver'],
                                                Horde::getDriverConfig('cache', $conf['cache']['driver']));

        /* Load attributes */
        $this->_loadAttributes();
    }

    /**
     * Attempts to return a reference to a concrete Faq_Categories instance.
     * It will only create a new instance if no Faq_Categories instance
     * currently exists.
     *
     * This method must be invoked as: $var = &Faq_Categories::singleton()
     *
     * @return object Faq_Categories  The concrete Faq_Categories reference,
     *                                 or false on an error.
     */
    function &singleton()
    {
        static $categories;

        if (!isset($categories)) {
            $categories = &new Faq_Categories();
        }

        return $categories;
    }

    /**
     * Returns a FaqCategories_DataTreeObject object of category data
     *
     * @param integer $id    An object id.
     */
    function getCategory($cid)
    {
        $category = &$this->_categories->getObjectById($cid, 'FaqCategories_DataTreeObject');
        if (is_a($category, 'PEAR_Error')) {
            return $category;
        }
        $category->setCategoriesOb($this);

        return $category;
    }

    /**
     * Remove a category by ID
     *
     * @param integer $cid  The ID of the category to remove.
     */
    function delete($cid)
    {
       $category = $this->_categories->getObjectById($cid);
       $result = $this->_categories->remove($category, true);
       if (is_a($result, 'PEAR_Error')) {
           return $result;
       }
       
       return $this->_expireCache();
    }

    /**
    * Save the category
    *
    * @param  mixed $info  The category data.
    *
    * @return mixed PEAR results.
    */
    function save($info)
    {
        if (empty($info['cid'])) {
            /* No existing category id, so this is a new category. */
            $category_name = microtime();
            if (!empty($info['parent_cid']) && $info['parent_cid'] != DATATREE_ROOT) {
                $parent_name = $this->_categories->getName($info['parent_cid']);
                $category_name = $parent_name . ':' . $category_name;
            }

            $category = &new FaqCategories_DataTreeObject($category_name);
            $category->setCategoriesOb($this);

            /* Add categories names into $data array which will be stored in
             * faq_categories_attributes table. */
            $category->data['n'] = $info['name'];
            $result = $this->_categories->add($category, true);
            if (is_a($result, 'PEAR_Error')) {
                return $result;
            }
            /* Get newly created category (for VFS category image) */
            $category = $this->getCategory($result);
        } else {
            /* Existing category id, so this is an update. */
            $category = $this->getCategory($info['cid']);
            /* Add categories names into $data array which will be stored in
             * faq_categories_attributes table. */
            $category->data['n'] = $info['name'];
            /* Delete old category image */
            if (!empty($info['current_image'])) {
                global $faq;
                $result = $faq->_vfs->deleteFile(FAQ_VFS_PATH . '/images/categories/', $info['current_image']);
                if (is_a($result, 'PEAR_Error')) {
                    return $result;
                }
            }
            /* Update category now */
            $result = $this->_categories->updateData($category);
            if (is_a($result, 'PEAR_Error')) {
                return $result;
            }
        }

        /* Write uploaded image into VFS */
        if (!empty($info['image'])) {
            $image_ext = MIME_Magic::MIMEToExt($info['image']['type']);
            $cat_image_filename =  time(). '.' . $image_ext;
            $category->data['p']['image'] = $cat_image_filename;
            $result = $this->_categories->updateData($category);
            if (is_a($result, 'PEAR_Error')) {
                return $result;
            }
            global $faq;
            $result = $faq->_vfs->write(FAQ_VFS_PATH . '/images/categories/', $cat_image_filename, $info['image']['file'], $autocreate = true);
            if (is_a($result, 'PEAR_Error')) {
                return $result;
            }
        }

        return $this->_expireCache();
    }

     /**
     * Return an array of data for edit form.
     *
     * @param  $cid  The category ID.
     *
     * @return mixed Array that hold data for edit form.
     */
    function getCatArray($cid)
    {
        $cat_array = array();
        $attributes = $this->_categories->getAttributes($cid);

        foreach ($attributes as $key => $attr) {
            if ($attributes[$key]['name'] == 'n') {
                $cat_array['name'][$attr['key']] = $attr['value'];
            }
            if ($attributes[$key]['name'] == 'p') {
                $cat_array['current_image'] = $attr['value'];
            }
        }

        $cat_array['cid'] = $cid;

        return $cat_array;
    }

    /**
     * Return a list of child categories.
     *
     * @param integer $cid  The root category id from which to start the list.
     *
     * @return string  An array with category id
     */
    function getChildList($cid = DATATREE_ROOT)
    {
        $root = $this->_categories->getName($cid);
        
        return $this->_categories->get(DATATREE_FORMAT_FLAT, $root, true);
    }

    /**
     * Return a list of available categories.
     *
     * @param integer $root       The root category id from which to start the
     *                            list.
     * @param boolean $formatted  Whether to format the list with indenting to
     *                            show hiearchy.
     * @param boolean $no_root    Should the root category be in the list or
     *                            not.
     *
     * @return string  An array with category id for keys and localised
     *                 category name for values.
     */
    function get($root = DATATREE_ROOT, $formatted = true, $no_root = false)
    {
        static $categories = array();
        if (!empty($categories)) {
            return $categories;
        }

        /* Get root category name and calculate base indent. */
        $root_name = $this->_categories->getName($root);
        $base_indent = substr_count($root_name, ':');

        $categories = $this->_categories->get(DATATREE_FORMAT_FLAT, $root, true);
        if ($no_root) {
            unset($categories[$root]);
            /* If there is an actual base indent, shorten it by one. */
            if ($base_indent > 0) {
                $base_indent--;
            }
        }

        $indent = '';
        foreach ($categories as $cid => $category) {
            if ($formatted) {
                $indent = substr_count($category, ':') - $base_indent;
                $indent = str_repeat('.. ', $indent);
            }
            $name = $this->getName($cid);
            $categories[$cid] = $indent . $name;
        }

        return $categories;
    }

    /**
     * Return a Horde_Tree representation of the Faq_Categories tree.
     *
     * @return string  The html showing the categories as a Horde_Tree.
     */
    function renderTree($current = DATATREE_ROOT, $click_url = null, $browse_only = false, $have_add_question = false)
    {
        require_once 'Horde/Tree.php';

        $nodes = $this->_getNodes();
        $icondir = array('icondir' => $GLOBALS['registry']->getImageDir());
        $cat_node = $icondir + array('icon' => 'folder_open.png');

        /* Set up the tree. */
        $tree = &Horde_Tree::singleton('faq_cats', 'javascript');
        $tree->setOption(array('width' => '100%', 'border' => '0', 'class' => 'question', 'cellpadding' => '0', 'cellspacing' => '0', 'alternate' => true));

        if (!$browse_only) {
            /* Print tree with editing buttons. */
            $add = Horde::applicationUrl('categories/edit.php');
            $edit = Horde::applicationUrl('categories/edit.php');
            $delete = Horde::applicationUrl('categories/delete.php');
            $edit_img = Horde::img('edit.png', _("Edit Category"), '', $GLOBALS['registry']->getImageDir('horde'));
            $delete_img = Horde::img('delete.png', _("Delete Category"), '', $GLOBALS['registry']->getImageDir('horde'));
            foreach ($nodes as $cid => $node) {
                $node_class = ($current == $cid) ? array('class' => 'selected') : array();
                $add_img = Horde::img('mkdir.png', _("Add New Category"));
                $add_link = Horde::link(Util::addParameter($add, 'parent_cid', $cid), _("Add New Category")) . $add_img . '</a>';
                if ($cid == DATATREE_ROOT) {
                    $base_node_params = $icondir + array('icon' => 'category.png');
                    $tree->addNode($cid, null, sprintf(_("%s categories"), $GLOBALS['registry']->get('name')), 0, true, $base_node_params + $node_class, array($add_link));
                } else {
                    $edit_link = Horde::link(Util::addParameter($edit, 'cid', $cid), _("Edit Category")) . $edit_img . '</a>';
                    $delete_link = Horde::link(Util::addParameter($delete, 'cid', $cid), _("Delete Category")) . $delete_img . '</a>';

                    $parent_id = $this->_categories->getParent($node);
                    $cat_extra = array($add_link, $edit_link, $delete_link);
                    $cat_name = $this->getName($cid);

                    /* If we have a url set the category name clickable. */
                    if (!empty($click_url)) {
                        $cat_name = Horde::link(Util::addParameter($click_url, 'cid', $cid), _("Select Category")) . $cat_name . '</a>';
                    }
                    $tree->addNode($cid, $parent_id, $cat_name, substr_count($node, ':') + 1, false, $cat_node + $node_class, $cat_extra);
                }
            }
        } else {
            /* Browse only mode. */
            $add_question = Horde::applicationUrl('questions/edit.php');

            foreach ($nodes as $cid => $node) {
                $node_class = ($current == $cid) ? array('class' => 'selected') : array();
                if ($cid == DATATREE_ROOT) {
                    $base_node_params = $icondir + array('icon' => 'category.png');
                    $tree->addNode($cid, null, sprintf(_("%s categories"), $GLOBALS['registry']->get('name')), 0, true, $base_node_params + $node_class);
                } else {
                    $add_img = Horde::img('mkdir.png', _("Add New Question"));
                    $add_question_link = Horde::link(Util::addParameter($add_question, 'cid', $cid), _("Add New Question")) . $add_img . '</a>';
                    $parent_id = $this->_categories->getParent($node);
                    $cat_name = $this->getName($cid);
                    /* If we have a url set the category name clickable. */
                    if (!empty($click_url)) {
                        $cat_name = Horde::link(Util::addParameter($click_url, 'cid', $cid), _("Select Category")) . $cat_name . '</a>';
                    }
                    if ($have_add_question) {
                        $tree->addNode($cid, $parent_id, $cat_name, substr_count($node, ':') + 1, false, $cat_node + $node_class, $add_question_link);
                    } else {
                        $tree->addNode($cid, $parent_id, $cat_name, substr_count($node, ':') + 1, false, $cat_node + $node_class);
                    }
                }
            }
        }

        return $tree->renderTree();
    }

    /**
     * Return a stored image for a category.
     *
     * @param integer $cid  The id of the category requested.
     * @param boolean $url  Return full image url.
     *
     * @return string  The image name.
     */
    function getImage($cid, $url = true)
    {
        /* Check if there is an image for requested category. */
        if (!isset($this->_attributes[$cid]['p']['image'])) {
            return '';
        }

        $img = $this->_attributes[$cid]['p']['image'];
        if (!$url) {
            return $img;
        }

        /* return url */
        if ($GLOBALS['conf']['images']['direct']) {
            $base_img_url = $GLOBALS['conf']['images']['direct'] . '/categories/' . $img;
        } else {
            $img_params = array('f' => $img,
                                's' => 'vfs',
                                'p' => FAQ_VFS_PATH . '/images/categories/',
                                'c' => 'faq');
            return Util::addParameter(Horde::url('/services/images/view.php'), $img_params);
        }
    }

    /**
     * Return a localised name for a category if available.
     * If no localised name return category ID
     *
     * @param integer $cid  The id of the category requested.
     *
     * @return string  The localised category name.
     */
    function getName($cid)
    {
        $lang = Faq::getLang();
        if (isset($this->_attributes[$cid]['n'][$lang])) {
            return $this->_attributes[$cid]['n'][$lang];
        } else {
            return '[' . $cid . ']';
        }
    }

     /**
     * Get category name with all prefixed parents names
     *
     * @param int $cid categoriy id
     *
     * @return mixed  True if the category exists, false otherwise.
     */
    function getFullName($cid)
    {
        $title = '';
        $nodes = $this->_getNodes();
        if (!isset($nodes[$cid])) {
            return $title;
        }

        foreach (explode(':', $nodes[$cid]) as $i => $id) {
            $title .= $this->getName($id) . ': ';
        }

        return substr($title, 0, -2);
    }

     /**
     * Get cached tree nodes
     *
     * @return mixed  True if the category exists, false otherwise.
     */
    function _getNodes()
    {
        if (($nodes = $this->_cache->get('faq_cat_nodes', $GLOBALS['conf']['cache']['default_lifetime']))) {
            $nodes = unserialize($nodes);
        } else {
            $nodes = $this->_categories->get(DATATREE_FORMAT_FLAT, DATATREE_ROOT, true);
            $this->_cache->set('faq_cat_nodes', serialize($nodes));
        }

        return $nodes;
    }

    /**
     * At startup load the attributes at for all categories into a
     * structured array for easy referencing of each attribute.
     *
     * @access private
     *
     * @return array  All the categories' attributes.
     */
    function _loadAttributes()
    {
        if (($attributes = $this->_cache->get('faq_cat_attributes', $GLOBALS['conf']['cache']['default_lifetime']))) {
            $this->_attributes = unserialize($attributes);
        }

        $raw_attributes = $this->_categories->getAttributes(array_keys($this->_getNodes()));
        if (is_a($raw_attributes, 'PEAR_Error')) {
            return $raw_attributes;
        }

        foreach ($raw_attributes as $cat_id => $attributes) {
            foreach ($attributes as $attr) {
                $this->_attributes[$cat_id][$attr['name']][$attr['key']] = $attr['value'];
            }
        }

        $this->_cache->set('faq_cat_attributes', serialize($this->_attributes));
    }

    /**
     * Expire categories cache data
     */
    function _expireCache()
    {
        return $this->_cache->expire('faq_cat_nodes') && 
               $this->_cache->expire('faq_cat_attributes');
    }

}

class FaqCategories_DataTreeObject extends DataTreeObject {

    /**
     * The Bookmarks object which this category came from - needed for
     * updating data in the backend to make changes stick, etc.
     *
     * @var categoryOb
     */
    var $_categoryOb;

    /**
     * The DataTreeObject_Bookmarks constructor. Just makes sure to call
     * the parent constructor so that the categories's name is set
     * properly.
     *
     * @param string $name The name of the category.
     */
    function FaqCategories_DataTreeObject($name)
    {
        parent::DataTreeObject($name);
    }

    /**
     * Associates a Bookmarks object with this category.
     *
     * @param object Boookmarks $bookmarksOb The Bookmarks object.
     */
    function setCategoriesOb(&$_categoryOb)
    {
        $this->_categoryOb = &$_categoryOb;
    }

    /**
     * Map this object's attributes from the data array into a format
     * that we can store in the attributes storage backend.
     *
     * @return array  The attributes array.
     */
    function _toAttributes()
    {
        $attributes = array();
        foreach ($this->data as $index => $entry) {
            foreach ($entry as $key => $value) {
                $attributes[] = array('name' => (string)$index,
                                      'key' => (string)$key,
                                      'value' => (string)$value);
            }
        }

        return $attributes;
    }

    /**
     * Take in a list of attributes from the backend and map it to our
     * internal data array.
     *
     * @param array $attributes  The list of attributes from the
     *                           backend (attribute name, key, and value).
     */
    function _fromAttributes($attributes)
    {
        $this->data = array();
        foreach ($attributes as $attr) {
            if (!isset($this->data[$attr['name']])) {
                $this->data[$attr['name']] = array();
            }
            $this->data[$attr['name']][$attr['key']] = $attr['value'];
        }
    }

}
