<?php
/**
 * Faq storage implementation for PHP's PEAR database abstraction layer.
 *
 * Required values for $params:<pre>
 *      'phptype'       The database type (e.g. 'pgsql', 'mysql', etc.).
 *      'charset'       The database's internal charset.</pre>
 *
 * Required by some database implementations:<pre>
 *      'database'      The name of the database.
 *      'hostspec'      The hostname of the database server.
 *      'protocol'      The communication protocol ('tcp', 'unix', etc.).
 *      'username'      The username with which to connect to the database.
 *      'password'      The password associated with 'username'.
 *      'options'       Additional options to pass to the database.
 *      'tty'           The TTY on which to connect to the database.
 *      'port'          The port on which to connect to the database.</pre>
 *
 * The table structure can be created by the scripts/drivers/faq_foo.sql
 * script.
 *
 * $Horde: faq/lib/Driver/sql.php,v 1.12 2006/03/07 18:17:34 slusarz Exp $
 *
 * @author  Duck <duck@obala.net>
 * @package Faq
 */
class Faq_Driver_sql extends Faq_Driver {

    /**
     * Hash containing connection parameters.
     *
     * @var array
     */
    var $_params = array();

    /**
     * Handle for the current database connection.
     *
     * @var DB
     */
    var $_db;

    /**
     * Handle for the current database connection, used for writing. Defaults
     * to the same handle as $_db if a separate write database is not required.
     *
     * @var DB
     */
    var $_write_db;

    /**
     * Boolean indicating whether or not we're connected to the SQL server.
     *
     * @var boolean
     */
    var $_connected = false;

    /**
     * Constructs a new SQL storage object.
     *
     * @param array $params  A hash containing connection parameters.
     */
    function Faq_Driver_sql($params = array())
    {
        $this->_params = $params;
        $this->_connect();
    }

    /**
     * Delete question
     */
    function delete($id)
    {
        $this->_db->query('DELETE FROM faq WHERE id=?', array($id));
        $this->_db->query('DELETE FROM faq_body WHERE id=?', array($id));
    }

    /**
     * Constructs a new SQL storage object.
     *
     * @param array $params  A hash containing connection parameters.
     */
    function save($info)
    {
        // get id and delete old data if needed
        if (!$info['id']) {
            $info['id'] = $this->_write_db->nextId('faq');
        } else {
            $this->delete($info['id']);
        }
    
        // insert categories
        foreach ($info['cids'] as $cid) {
            $result = $this->_db->query('INSERT INTO faq (id, cid) VALUES(?, ?)', array($info['id'], $cid));
            if (is_a($result, 'PEAR_Error')) {
                return $result;
            }
        }

        // insert content
        foreach (array_keys($info['question']) as $lang) {
            $query = 'INSERT INTO faq_body (id, lang, question, answer) VALUES(?, ?, ?, ?)';
            $values = array($info['id'], $lang, $info['question'][$lang], $info['answer'][$lang]);
            $result = $this->_db->query($query, $values);
            if (is_a($result, 'PEAR_Error')) {
                return $result;
            }
        }

        return true;
    }

    /**
     * Attempts to open a persistent connection to the SQL server.
     *
     * @return boolean  True on success; exits (Horde::fatal()) on error.
     */
    function getAll($cid = false, $full = false)
    {
        $values = array(Faq::getLang());
        $query = 'SELECT DISTINCT a.id, b.question ';

        if ($full) {
            $query .= ', b.answer ';
        }

        $query .= 'FROM faq AS a, faq_body AS b WHERE a.id=b.id AND b.lang=?';

        if ($cid) {
            $query .= ' AND cid=? ';
            $values[] = $cid;
        }

        $query .= ' ORDER BY b.question ASC';
  
        return $this->_db->getAll($query, $values, DB_FETCHMODE_ASSOC);
    }

    /**
     * Get all data of a question
     */
    function getOne($id)
    {
        $question = $this->_db->getAssoc('SELECT lang, question, answer FROM faq_body WHERE id=?', false, array($id), DB_FETCHMODE_ASSOC);
        if (is_a($question, 'PEAR_Error')) {
            return $question;
        }

        $result = $this->_db->getCol('SELECT cid FROM faq WHERE id=?', 0, array($id));
        if (is_a($result, 'PEAR_Error')) {
            return $result;
        }

        $question['cids'] = $result;

        return $question;
    }

    /**
     * Attempts to open a persistent connection to the SQL server.
     *
     * @return boolean  True on success; exits (Horde::fatal()) on error.
     */
    function _connect()
    {
        if ($this->_connected) {
            return true;
        }

        Horde::assertDriverConfig($this->_params, 'storage',
                                  array('phptype', 'charset'));

        if (!isset($this->_params['database'])) {
            $this->_params['database'] = '';
        }
        if (!isset($this->_params['username'])) {
            $this->_params['username'] = '';
        }
        if (!isset($this->_params['hostspec'])) {
            $this->_params['hostspec'] = '';
        }

        /* Connect to the SQL server using the supplied parameters. */
        require_once 'DB.php';
        $this->_write_db = &DB::connect($this->_params,
                                        array('persistent' => !empty($this->_params['persistent'])));
        if (is_a($this->_write_db, 'PEAR_Error')) {
            Horde::fatal($this->_write_db, __FILE__, __LINE__);
        }

        // Set DB portability options.
        switch ($this->_write_db->phptype) {
        case 'mssql':
            $this->_write_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS | DB_PORTABILITY_RTRIM);
            break;
        default:
            $this->_write_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);
        }

        /* Check if we need to set up the read DB connection seperately. */
        if (!empty($this->_params['splitread'])) {
            $params = array_merge($this->_params, $this->_params['read']);
            $this->_db = &DB::connect($params,
                                      array('persistent' => !empty($params['persistent'])));
            if (is_a($this->_db, 'PEAR_Error')) {
                Horde::fatal($this->_db, __FILE__, __LINE__);
            }

            // Set DB portability options.
            switch ($this->_db->phptype) {
            case 'mssql':
                $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS | DB_PORTABILITY_RTRIM);
                break;
            default:
                $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);
            }

        } else {
            /* Default to the same DB handle for the writer too. */
            $this->_db =& $this->_write_db;
        }

        $this->_connected = true;

        return true;
    }

    /**
     * Disconnects from the SQL server and cleans up the connection.
     *
     * @return boolean  True on success, false on failure.
     */
    function _disconnect()
    {
        if ($this->_connected) {
            $this->_connected = false;
            $this->_db->disconnect();
            $this->_write_db->disconnect();
        }

        return true;
    }

}
